@@ -54,10 +54,11 @@ capture_config_t capture_cfg =
 { 0 };
 
 void
-capture_init(size_t limit, bool rtp_capture)
+capture_init(size_t limit, bool rtp_capture, bool rotate)
 {
     capture_cfg.limit = limit;
     capture_cfg.rtp_capture = rtp_capture;
+    capture_cfg.rotate = rotate;
     capture_cfg.sources = vector_create(1, 1);
     capture_cfg.tcp_reasm = vector_create(0, 10);
     capture_cfg.ip_reasm = vector_create(0, 10);
@@ -127,12 +128,15 @@ capture_online(const char *dev, const char *outfile)
     }
 
     // Open capture device
-    capinfo->handle = pcap_open_live(dev, BUFSIZ, 1, 1000, errbuf);
+    capinfo->handle = pcap_open_live(dev, MAXIMUM_SNAPLEN, 1, 1000, errbuf);
     if (capinfo->handle == NULL) {
         fprintf(stderr, "Couldn't open device %s: %s\n", dev, errbuf);
         return 2;
     }
 
+    // Store capture device
+    capinfo->device = dev;
+
     // Get datalink to parse packets correctly
     capinfo->link = pcap_datalink(capinfo->handle);
 
@@ -236,8 +240,12 @@ parse_packet(u_char *info, const struct pcap_pkthdr *header, const u_char *packe
         return;
 
     // Check if we have reached capture limit
-    if (capture_cfg.limit && sip_calls_count() >= capture_cfg.limit)
-        return;
+    if (capture_cfg.limit && sip_calls_count() >= capture_cfg.limit) {
+        // If capture rotation is disabled, just skip this packet
+        if (!capture_cfg.rotate) {
+            return;
+        }
+    }
 
     // Check maximum capture length
     if (header->caplen > MAX_CAPTURE_LEN)
@@ -464,17 +472,27 @@ capture_packet_reasm_ip(capture_info_t *capinfo, const struct pcap_pkthdr *heade
         pkt = packet_create(ip_ver, ip_proto, src, dst, ip_id);
         packet_add_frame(pkt, header, packet);
         vector_append(capture_cfg.ip_reasm, pkt);
-        return NULL;
     }
 
-    // If no more fragments
+    // Add this IP content length to the total captured of the packet
+    pkt->ip_cap_len += ip_len;
+
+    // Calculate how much data we need to complete this packet
+    // The total packet size can only be known using the last fragment of the packet
+    // where 'No more fragments is enabled' and it's calculated based on the
+    // last fragment offset
     if ((ip_off & IP_MF) == 0) {
+        pkt->ip_exp_len = ip_frag_off + ip_len + ip_hl;
+    }
+
+    // If we have the whole packet (captured length is expected length)
+    if (pkt->ip_cap_len == pkt->ip_exp_len) {
         // TODO Dont check the flag, check the holes
         // Calculate assembled IP payload data
         it = vector_iterator(pkt->frames);
         while ((frame = vector_iterator_next(&it))) {
             struct ip *frame_ip = (struct ip *) (frame->data + link_hl);
-            len_data += frame->header->caplen - link_hl - frame_ip->ip_hl * 4;
+            len_data += ntohs(frame_ip->ip_len) - frame_ip->ip_hl * 4;
         }
 
         // Check packet content length
@@ -490,7 +508,7 @@ capture_packet_reasm_ip(capture_info_t *capinfo, const struct pcap_pkthdr *heade
             struct ip *frame_ip = (struct ip *) (frame->data + link_hl);
             memcpy(packet + link_hl + ip_hl + (ntohs(frame_ip->ip_off) & IP_OFFMASK) * 8,
                    frame->data + link_hl + frame_ip->ip_hl * 4,
-                   frame->header->caplen - link_hl - frame_ip->ip_hl * 4);
+                   ntohs(frame_ip->ip_len) - frame_ip->ip_hl * 4);
         }
 
         *caplen = link_hl + ip_hl + len_data;
@@ -798,9 +816,19 @@ capture_set_bpf_filter(const char *filter)
 
     }
 
+    // Store valid capture filter
+    capture_cfg.filter = filter;
+
     return 0;
 }
 
+const char *
+capture_get_bpf_filter()
+{
+    return capture_cfg.filter;
+}
+
+
 void
 capture_set_paused(int pause)
 {
@@ -852,7 +880,18 @@ capture_input_file()
     } else {
         return "Multiple files";
     }
+}
 
+const char *
+capture_device()
+{
+    capture_info_t *capinfo;
+
+    if (vector_count(capture_cfg.sources) == 1) {
+        capinfo = vector_first(capture_cfg.sources);
+        return capinfo->device;
+    }
+    return NULL;
 }
 
 const char*
