@@ -74,6 +74,8 @@
 
 //! Max allowed packet assembled size
 #define MAX_CAPTURE_LEN 20480
+//! Max allowed packet length
+#define MAXIMUM_SNAPLEN 262144
 
 //! Define VLAN 802.1Q Ethernet type
 #ifndef ETHERTYPE_8021Q
@@ -119,10 +121,14 @@ struct capture_config {
     size_t limit;
     //! Also capture RTP packets
     bool rtp_capture;
+    //! Rotate capturad dialogs when limit have reached
+    bool rotate;
     //! Where should we store captured packets
     enum capture_storage storage;
     //! Key file for TLS decrypt
     const char *keyfile;
+    //! capture filter expression text
+    const char *filter;
     //! The compiled filter expression
     struct bpf_program fp;
     //! libpcap dump file handler
@@ -156,6 +162,8 @@ struct capture_info
     bpf_u_int32 net;
     //! Input file in Offline capture
     const char *infile;
+    //! Capture device in Online mode
+    const char *device;
     //! Capture thread for online capturing
     pthread_t capture_t;
 };
@@ -165,9 +173,10 @@ struct capture_info
  *
  * @param limit Numbers of calls >0
  * @param rtp_catpure Enable rtp capture
+ * @param rotate Enable capture rotation
  */
 void
-capture_init(size_t limit, bool rtp_capture);
+capture_init(size_t limit, bool rtp_capture, bool rotate);
 
 /**
  * @brief Deinitialize capture data
@@ -316,6 +325,14 @@ capture_is_online();
 int
 capture_set_bpf_filter(const char *filter);
 
+/**
+ * @brief Get the configured BPF filter
+ *
+ * @return String containing the BPF filter text or NULL
+ */
+const char *
+capture_get_bpf_filter();
+
 /**
  * @brief Pause/Resume capture
  *
@@ -353,6 +370,15 @@ capture_status_desc();
 const char*
 capture_input_file();
 
+/**
+ * @brief Get Device interface from Online mode
+ *
+ * @return Device name used to capture packets
+ * @return NULL in Offline or Mixed mode
+ */
+const char *
+capture_device();
+
 /**
  * @brief Get Key file from decrypting TLS packets
  *
