@@ -42,7 +42,7 @@
 #define __SNGREP_SETTING_H
 
 //! Max setting value
-#define MAX_SETTING_LEN   80
+#define MAX_SETTING_LEN   1024
 
 //! Shorter declarartion of setting_option struct
 typedef struct setting_option setting_t;
@@ -79,6 +79,7 @@ enum setting_id {
     SETTING_CAPTURE_KEYFILE,
     SETTING_CAPTURE_RTP,
     SETTING_CAPTURE_STORAGE,
+    SETTING_CAPTURE_ROTATE,
     SETTING_SIP_NOINCOMPLETE,
     SETTING_SIP_CALLS,
     SETTING_SAVEPATH,
@@ -97,17 +98,21 @@ enum setting_id {
     SETTING_CF_MEDIA,
     SETTING_CF_DELTA,
     SETTING_CR_SCROLLSTEP,
+    SETTING_FILTER_PAYLOAD,
     SETTING_FILTER_METHODS,
+#ifdef USE_EEP
     SETTING_EEP_SEND,
     SETTING_EEP_SEND_VER,
     SETTING_EEP_SEND_ADDR,
     SETTING_EEP_SEND_PORT,
     SETTING_EEP_SEND_PASS,
+    SETTING_EEP_SEND_ID,
     SETTING_EEP_LISTEN,
     SETTING_EEP_LISTEN_VER,
     SETTING_EEP_LISTEN_ADDR,
     SETTING_EEP_LISTEN_PORT,
     SETTING_EEP_LISTEN_PASS,
+#endif
     SETTING_COUNT
 };
 
