@@ -31,7 +31,6 @@
 #include <stdio.h>
 #include <time.h>
 #include <pthread.h>
-#include <search.h>
 #include <stdarg.h>
 #include "sip.h"
 #include "option.h"
@@ -146,6 +145,7 @@ sip_init(int limit, int only_calls, int no_incomplete)
     calls.limit = limit;
     calls.only_calls = only_calls;
     calls.ignore_incomplete = no_incomplete;
+    calls.last_index = 0;
 
     // Create a vector to store calls
     calls.list = vector_create(200, 50);
@@ -154,7 +154,7 @@ sip_init(int limit, int only_calls, int no_incomplete)
     calls.active = vector_create(10, 10);
 
     // Create hash table for callid search
-    hcreate(calls.limit);
+    calls.callids = htable_create(calls.limit);
 
     // By default sort by call index ascending
     calls.sort.by = SIP_ATTR_CALLINDEX;
@@ -162,16 +162,18 @@ sip_init(int limit, int only_calls, int no_incomplete)
 
     // Initialize payload parsing regexp
     match_flags = REG_EXTENDED | REG_ICASE | REG_NEWLINE;
-    regcomp(&calls.reg_method, "^([a-zA-Z]+) sips?:[^ ]+ SIP/2.0\r", match_flags & ~REG_NEWLINE);
+    regcomp(&calls.reg_method, "^([a-zA-Z]+) [a-zA-Z]+:[^ ]+ SIP/2.0\r", match_flags & ~REG_NEWLINE);
     regcomp(&calls.reg_callid, "^(Call-ID|i):[ ]*([^ ]+)\r$", match_flags);
     regcomp(&calls.reg_xcallid, "^(X-Call-ID|X-CID):[ ]*([^ ]+)\r$", match_flags);
-    regcomp(&calls.reg_response, "^SIP/2.0[ ]*(([0-9]{3}) [^\r]+)\r", match_flags & ~REG_NEWLINE);
+    regcomp(&calls.reg_response, "^SIP/2.0[ ]*(([0-9]{3}) [^\r]*)\r", match_flags & ~REG_NEWLINE);
     regcomp(&calls.reg_cseq, "^CSeq:[ ]*([0-9]+) .+\r$", match_flags);
     regcomp(&calls.reg_from, "^(From|f):[ ]*[^:]*:(([^@]+)@?[^\r>;]+)", match_flags);
     regcomp(&calls.reg_to, "^(To|t):[ ]*[^:]*:(([^@]+)@?[^\r>;]+)", match_flags);
-    regcomp(&calls.reg_valid, "^([A-Z]+ sip:|SIP/2.0 [0-9]{3})", match_flags & ~REG_NEWLINE);
+    regcomp(&calls.reg_valid, "^([A-Z]+ [a-zA-Z]+:|SIP/2.0 [0-9]{3})", match_flags & ~REG_NEWLINE);
     regcomp(&calls.reg_cl, "^(Content-Length|l):[ ]*([0-9]+)\r$", match_flags);
     regcomp(&calls.reg_body, "\r\n\r\n(.*)", match_flags & ~REG_NEWLINE);
+    regcomp(&calls.reg_reason, "Reason:[ ]*[^\r]*;text=\"([^\r]+)\"", match_flags);
+    regcomp(&calls.reg_warning, "Warning:[ ]*([0-9]*)", match_flags);
 
 }
 
@@ -181,7 +183,7 @@ sip_deinit()
     // Remove all calls
     sip_calls_clear();
     // Remove Call-id hash table
-    hdestroy();
+    htable_destroy(calls.callids);
     // Remove calls vector
     vector_destroy(calls.list);
     vector_destroy(calls.active);
@@ -196,6 +198,8 @@ sip_deinit()
     regfree(&calls.reg_valid);
     regfree(&calls.reg_cl);
     regfree(&calls.reg_body);
+    regfree(&calls.reg_reason);
+    regfree(&calls.reg_warning);
 }
 
 
@@ -277,7 +281,6 @@ sip_validate_packet(packet_t *packet)
 sip_msg_t *
 sip_check_packet(packet_t *packet)
 {
-    ENTRY entry;
     sip_msg_t *msg;
     sip_call_t *call;
     char callid[1024], xcallid[1024];
@@ -337,17 +340,19 @@ sip_check_packet(packet_t *packet)
         // Get the Call-ID of this message
         sip_get_xcallid((const char*) payload, xcallid);
 
+        // Rotate call list if limit has been reached
+        if (calls.limit == sip_calls_count())
+            sip_calls_rotate();
+
         // Create the call if not found
         if (!(call = call_create(callid, xcallid)))
             goto skip_message;
 
-        // Store this call in hash table
-        entry.key = (char *) call->callid;
-        entry.data = (void *) call;
-        hsearch(entry, ENTER);
+        // Add this Call-Id to hash table
+        htable_insert(calls.callids, call->callid, call);
 
         // Set call index
-        call->index = vector_count(calls.list) + 1;
+        call->index = ++calls.last_index;
 
         // Mark this as a new call
         newcall = true;
@@ -369,18 +374,23 @@ sip_check_packet(packet_t *packet)
     // Add the message to the call
     call_add_message(call, msg);
 
+    // check if message is a retransmission
+    call_msg_retrans_check(msg);
+
     if (call_is_invite(call)) {
         // Parse media data
         sip_parse_msg_media(msg, payload);
         // Update Call State
         call_update_state(call, msg);
+        // Parse extra fields
+        sip_parse_extra_headers(msg, payload);
         // Check if this call should be in active call list
         if (call_is_active(call)) {
-            if (vector_index(calls.active, call) == -1) {
+            if (sip_call_is_active(call)) {
                 vector_append(calls.active, call);
             }
         } else {
-            if (vector_index(calls.active, call) != -1) {
+            if (sip_call_is_active(call)) {
                 vector_remove(calls.active, call);
             }
         }
@@ -430,6 +440,12 @@ sip_active_calls_iterator()
     return vector_iterator(calls.active);
 }
 
+bool
+sip_call_is_active(sip_call_t *call)
+{
+    return vector_index(calls.active, call) != -1;
+}
+
 vector_t *
 sip_calls_vector()
 {
@@ -459,13 +475,7 @@ sip_find_by_index(int index)
 sip_call_t *
 sip_find_by_callid(const char *callid)
 {
-    ENTRY entry, *eptr;
-
-    entry.key = (char *) callid;
-    if ((eptr = hsearch(entry, FIND)))
-        return eptr->data;
-
-    return NULL;
+    return htable_find(calls.callids, callid);
 }
 
 int
@@ -547,12 +557,20 @@ sip_parse_msg_payload(sip_msg_t *msg, const u_char *payload)
     if (regexec(&calls.reg_from, (const char *)payload, 4, pmatch, 0) == 0) {
         msg->sip_from = sng_malloc((int)pmatch[2].rm_eo - pmatch[2].rm_so + 1);
         strncpy(msg->sip_from, (const char *)payload +  pmatch[2].rm_so, (int)pmatch[2].rm_eo - pmatch[2].rm_so);
+    } else {
+        // Malformed From Header
+        msg->sip_from = sng_malloc(12);
+        strncpy(msg->sip_from, "<malformed>", 11);
     }
 
     // To
     if (regexec(&calls.reg_to, (const char *)payload, 4, pmatch, 0) == 0) {
         msg->sip_to = sng_malloc((int)pmatch[2].rm_eo - pmatch[2].rm_so + 1);
         strncpy(msg->sip_to, (const char *)payload +  pmatch[2].rm_so, (int)pmatch[2].rm_eo - pmatch[2].rm_so);
+    } else {
+        // Malformed To Header
+        msg->sip_to = sng_malloc(12);
+        strncpy(msg->sip_to, "<malformed>", 11);
     }
 
     return 0;
@@ -561,7 +579,19 @@ sip_parse_msg_payload(sip_msg_t *msg, const u_char *payload)
 void
 sip_parse_msg_media(sip_msg_t *msg, const u_char *payload)
 {
+
+#define ADD_STREAM(stream) \
+    if (stream) { \
+        if (!rtp_find_call_stream(call, src, stream->dst)) { \
+          call_add_stream(call, stream); \
+      } else { \
+          sng_free(stream); \
+          stream = NULL; \
+      } \
+    }
+
     address_t dst, src = { };
+    rtp_stream_t *rtp_stream = NULL, *rtcp_stream = NULL, *msg_rtp_stream = NULL;
     char media_type[MEDIATYPELEN] = { };
     char media_format[30] = { };
     uint32_t media_fmt_pref;
@@ -570,12 +600,25 @@ sip_parse_msg_media(sip_msg_t *msg, const u_char *payload)
     char *payload2, *tofree, *line;
     sip_call_t *call = msg_get_call(msg);
 
+    // If message is retrans, there's no need to parse the payload again
+    if (msg->retrans) {
+        // Use the media vector from the original message
+        msg->medias = msg->retrans->medias;
+        return;
+    }
+
     // Parse each line of payload looking for sdp information
     tofree = payload2 = strdup((char*)payload);
     while ((line = strsep(&payload2, "\r\n")) != NULL) {
         // Check if we have a media string
         if (!strncmp(line, "m=", 2)) {
             if (sscanf(line, "m=%s %hu RTP/%*s %u", media_type, &dst.port, &media_fmt_pref) == 3) {
+
+                // Add streams from previous 'm=' line to the call
+                ADD_STREAM(msg_rtp_stream);
+                ADD_STREAM(rtp_stream);
+                ADD_STREAM(rtcp_stream);
+
                 // Create a new media structure for this message
                 if ((media = media_create(msg))) {
                     media_set_type(media, media_type);
@@ -589,15 +632,18 @@ sip_parse_msg_media(sip_msg_t *msg, const u_char *payload)
                      * and port of the stream.
                      */
                     // Create a new stream with this destination address:port
-                    if (!call_msg_is_retrans(msg)) {
-                        if (!rtp_find_call_stream(call, src, dst)) {
-                            // Create RTP stream
-                            call_add_stream(call, stream_create(media, dst, PACKET_RTP));
-                            // Create early RTCP stream
-                            dst.port++;
-                            call_add_stream(call, stream_create(media, dst, PACKET_RTCP));
-                        }
-                    }
+
+                    // Create RTP stream with source of message as destination address
+                    msg_rtp_stream = stream_create(media, dst, PACKET_RTP);
+                    msg_rtp_stream->dst = msg->packet->src;
+                    msg_rtp_stream->dst.port = dst.port;
+
+                    // Create RTP stream
+                    rtp_stream = stream_create(media, dst, PACKET_RTP);
+
+                    // Create RTCP stream
+                    rtcp_stream = stream_create(media, dst, PACKET_RTCP);
+                    rtcp_stream->dst.port++;
                 }
             }
         }
@@ -606,6 +652,8 @@ sip_parse_msg_media(sip_msg_t *msg, const u_char *payload)
         if (!strncmp(line, "c=", 2)) {
             if (sscanf(line, "c=IN IP4 %s", dst.ip) && media) {
                 media_set_address(media, dst);
+                strcpy(rtp_stream->dst.ip, dst.ip);
+                strcpy(rtcp_stream->dst.ip, dst.ip);
             }
         }
 
@@ -617,32 +665,70 @@ sip_parse_msg_media(sip_msg_t *msg, const u_char *payload)
         }
 
         // Check if we have attribute format RTCP port
-        if (!strncmp(line, "a=rtcp:", 7)) {
-            if (media && sscanf(line, "a=rtcp:%hu", &dst.port)) {
-                // Create early RTCP stream
-                if (!rtp_find_call_stream(call, src, dst)) {
-                    call_add_stream(call, stream_create(media, dst, PACKET_RTCP));
-                }
-            }
+        if (!strncmp(line, "a=rtcp:", 7) && rtcp_stream) {
+            sscanf(line, "a=rtcp:%hu", &rtcp_stream->dst.port);
         }
 
 
     }
+
+    // Add streams from last 'm=' line to the call
+    ADD_STREAM(msg_rtp_stream);
+    ADD_STREAM(rtp_stream);
+    ADD_STREAM(rtcp_stream);
+
     sng_free(tofree);
+
+#undef ADD_STREAM
 }
 
+void
+sip_parse_extra_headers(sip_msg_t *msg, const u_char *payload)
+{
+    regmatch_t pmatch[4];
+    char warning[10];
+
+     // Reason text
+     if (regexec(&calls.reg_reason, (const char *)payload, 2, pmatch, 0) == 0) {
+         msg->call->reasontxt = sng_malloc((int)pmatch[1].rm_eo - pmatch[1].rm_so + 1);
+         strncpy(msg->call->reasontxt, (const char *)payload +  pmatch[1].rm_so, (int)pmatch[1].rm_eo - pmatch[1].rm_so);
+     }
+
+     // Warning code
+     if (regexec(&calls.reg_warning, (const char *)payload, 2, pmatch, 0) == 0) {
+         strncpy(warning, (const char *)payload +  pmatch[1].rm_so, (int)pmatch[1].rm_eo - pmatch[1].rm_so);
+         msg->call->warning = atoi(warning);
+     }
+}
 
 void
 sip_calls_clear()
 {
     // Create again the callid hash table
-    hdestroy();
-    hcreate(calls.limit);
+    htable_destroy(calls.callids);
+    calls.callids = htable_create(calls.limit);
     // Remove all items from vector
     vector_clear(calls.list);
     vector_clear(calls.active);
 }
 
+void
+sip_calls_rotate()
+{
+    sip_call_t *call;
+    vector_iter_t it = vector_iterator(calls.list);
+    while ((call = vector_iterator_next(&it))) {
+        if (!call->locked) {
+            // Remove from callids hash
+            htable_remove(calls.callids, call->callid);
+            // Remove first call from active and call lists
+            vector_remove(calls.active, call);
+            vector_remove(calls.list, call);
+            return;
+        }
+    }
+}
+
 int
 sip_set_match_expression(const char *expr, int insensitive, int invert)
 {
@@ -674,6 +760,12 @@ sip_set_match_expression(const char *expr, int insensitive, int invert)
 #endif
 }
 
+const char *
+sip_get_match_expression()
+{
+    return calls.match_expr;
+}
+
 int
 sip_check_match_expression(const char *payload)
 {
